<?php

declare(strict_types=1);

namespace Drupal\notifier_chat_channel;

use Drupal\notifier_chat_channel\Transport\Slack\EventListener\SlackRecipients;
use Drupal\notifier_chat_channel\Transport\Slack\SlackFieldConfiguration;
use Symfony\Component\DependencyInjection\ChildDefinition;
use Symfony\Component\DependencyInjection\Compiler\CompilerPassInterface;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\DependencyInjection\Reference;
use Symfony\Component\Mercure\HubRegistry;
use Symfony\Component\Notifier\Bridge;

/**
 * Notifier Chat Channel compiler pass.
 *
 * @phpstan-import-type SlackFieldMapping from \Drupal\notifier_chat_channel\Transport\Slack\SlackFieldConfiguration
 */
final class NotifierChatChannelCompilerPass implements CompilerPassInterface {

  public function process(ContainerBuilder $container): void {
    $chatterFactories = [
      // @phpcs:disable
      'bluesky' => Bridge\Bluesky\BlueskyTransportFactory::class,
      'chatwork' => Bridge\Chatwork\ChatworkTransportFactory::class,
      'discord' => Bridge\Discord\DiscordTransportFactory::class,
      'fake-chat' => Bridge\FakeChat\FakeChatTransportFactory::class,
      'firebase' => Bridge\Firebase\FirebaseTransportFactory::class,
      'google-chat' => Bridge\GoogleChat\GoogleChatTransportFactory::class,
      'line-bot' => Bridge\LineBot\LineBotTransportFactory::class,
      'line-notify' => Bridge\LineNotify\LineNotifyTransportFactory::class,
      'linked-in' => Bridge\LinkedIn\LinkedInTransportFactory::class,
      'mastodon' => Bridge\Mastodon\MastodonTransportFactory::class,
      'mattermost' => Bridge\Mattermost\MattermostTransportFactory::class,
      'mercure' => Bridge\Mercure\MercureTransportFactory::class,
      'microsoft-teams' => Bridge\MicrosoftTeams\MicrosoftTeamsTransportFactory::class,
      'pager-duty' => Bridge\PagerDuty\PagerDutyTransportFactory::class,
      'rocket-chat' => Bridge\RocketChat\RocketChatTransportFactory::class,
      'slack' => Bridge\Slack\SlackTransportFactory::class,
      'telegram' => Bridge\Telegram\TelegramTransportFactory::class,
      'twitter' => Bridge\Twitter\TwitterTransportFactory::class,
      'zendesk' => Bridge\Zendesk\ZendeskTransportFactory::class,
      'zulip' => Bridge\Zulip\ZulipTransportFactory::class,
      // @phpcs:enable
    ];

    if (!$container->hasDefinition(HubRegistry::class)) {
      unset($chatterFactories['mercure']);
    }

    $parentPackages = ['symfony/framework-bundle', 'symfony/notifier'];

    foreach ($chatterFactories as $name => $class) {
      if (FALSE === ContainerBuilder::willBeAvailable(\sprintf('symfony/%s-notifier', $name), $class, $parentPackages)) {
        continue;
      }

      $factory = (new ChildDefinition('notifier.transport_factory.abstract'))
        ->setClass($class)
        ->addTag('chatter.transport_factory')
        ->setAutowired(TRUE)
        ->setAutoconfigured(TRUE);

      if ($name === 'fake-chat') {
        $factory->setArgument('$logger', new Reference('logger.channel.chat.fake'));
      }

      $container->setDefinition(\sprintf('notifier.transport_factory.%s', $name), $factory);
    }

    // If Fake was not defined, then remove the logger defined in services.yml.
    if (FALSE === $container->hasDefinition('notifier.transport_factory.fake-chat')) {
      $container->removeDefinition('logger.channel.chat.fake');
    }

    // Transport-specific recipient listeners.
    /** @var array<array{transport: string}> $mappingRaw */
    $mappingRaw = $container->getParameter('notifier_chat_channel.field_mapping');
    $mappingByTransport = [];
    foreach ($mappingRaw as $map) {
      ['transport' => $transport] = $map;
      if ($container->hasDefinition(\sprintf('notifier.transport_factory.%s', $transport))) {
        unset($map['transport']);
        $mappingByTransport[$transport][] = $map;
      }
    }

    if (FALSE === $container->hasDefinition('notifier.transport_factory.slack') || FALSE === \array_key_exists('slack', $mappingRaw)) {
      $container->removeDefinition(SlackRecipients::class);
    }
    else {
      $container
        ->getDefinition(SlackRecipients::class)
        ->setArgument('$configurations', \serialize(\array_map(
          static function (mixed $map): SlackFieldConfiguration {
            /** @var SlackFieldMapping $map */
            return SlackFieldConfiguration::fromRaw($map);
          },
          $mappingByTransport['slack'],
        )));
    }
  }

}
