<?php

declare(strict_types=1);

namespace Drupal\Tests\notifier_chat_channel\Kernel;

use Drupal\Core\DependencyInjection\ContainerBuilder;
use Drupal\entity_test\Entity\EntityTest;
use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\KernelTests\KernelTestBase;
use Drupal\notifier\Sender\SenderInterface;
use Drupal\notifier_chat_channel\Transport\Slack\Recipient\SlackRecipientInterface;
use Ramsey\Collection\Collection;
use Symfony\Component\HttpClient\MockHttpClient;
use Symfony\Component\Notifier\Bridge\Slack\SlackOptions;
use Symfony\Component\Notifier\Message\ChatMessage;
use Symfony\Component\Notifier\Notification\ChatNotificationInterface;
use Symfony\Component\Notifier\Notification\Notification;
use Symfony\Component\Notifier\NotifierInterface;
use Symfony\Component\Notifier\Recipient\RecipientInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;
use Symfony\Contracts\HttpClient\ResponseInterface;

/**
 * Tests Notifier Chat Channel - Slack.
 */
final class NotifierChatChannelSlackTest extends KernelTestBase {

  private \Closure $responseFactory;

  protected static $modules = [
    'entity_test',
    'field',
    'notifier',
    'notifier_chat_channel',
    'symfony_http_client',
    'user',
  ];

  protected function setUp(): void {
    parent::setUp();
    $this->installEntitySchema('entity_test');
    FieldStorageConfig::create([
      'field_name' => 'field_slack',
      'type' => 'string',
      'entity_type' => 'entity_test',
    ])->save();
    FieldConfig::create([
      'field_name' => 'field_slack',
      'entity_type' => 'entity_test',
      'bundle' => 'entity_test',
    ])->save();
  }

  /**
   * Tests sending a Slack message directly with Notifier service.
   */
  public function testNotifier(): void {
    $notification = (new Notification())
      ->subject('Test message!')
      ->channels(['chat/my_slack']);

    $requests = $this->requestRecorder();
    static::notifier()->send(notification: $notification);

    static::assertCount(1, $requests);
    /** @var array{text: mixed, blocks: mixed, channel: string} $body */
    $body = \Safe\json_decode($requests[0][2]['body'], associative: TRUE);
    static::assertEquals('Test message!', $body['text']);
    static::assertEquals('general', $body['channel']);
  }

  /**
   * Tests sending a Slack message via Sender and field mapping.
   */
  public function testFieldMapping(): void {
    $notification = new class extends Notification implements ChatNotificationInterface {

      public function asChatMessage(RecipientInterface $recipient, ?string $transport = NULL): ?ChatMessage {
        $options = (new SlackOptions());
        $recipient instanceof SlackRecipientInterface
          ? $recipient->addRecipientToOptions($options)
          : throw new \LogicException('Unexpected recipient of ' . $recipient::class);
        return new ChatMessage('Test Slack message!!', options: $options);
      }

    };

    $notification->subject('Test Slack message!!');
    $notification->channels(['chat/my_slack']);

    $for = EntityTest::create([
      'field_slack' => '@my_slack_username',
    ]);

    $requests = $this->requestRecorder();
    static::sender()->send(for: $for, notification: $notification);

    static::assertCount(1, $requests);
    /** @var array{text: mixed, blocks: mixed, channel: string} $body */
    $body = \Safe\json_decode($requests[0][2]['body'], associative: TRUE);
    static::assertEquals('Test Slack message!!', $body['text']);
    static::assertEquals('@my_slack_username', $body['channel']);
  }

  public function register(ContainerBuilder $container): void {
    parent::register($container);

    $container->setParameter('chatter.transports', [
      'my_slack' => 'slack://xoxb-abc123@default?channel=general',
    ]);
    $container->setParameter('notifier_chat_channel.field_mapping', [
      'slack' => [
        'transport' => 'slack',
        'entity_type' => 'entity_test',
        'bundle' => 'entity_test',
        'field_name' => 'field_slack',
        'prefix' => [
          'non_prefixed_as' => 'user',
          'user_enabled' => TRUE,
          'channel_enabled' => TRUE,
        ],
      ],
    ]);

    $container->set(HttpClientInterface::class, new MockHttpClient(responseFactory: fn($method, $url, $options): ResponseInterface => ($this->responseFactory)($method, $url, $options)));
  }

  /**
   * @phpstan-return \Ramsey\Collection\Collection<array{string, string, array{body: string}}>
   */
  private function requestRecorder(): Collection {
    /** @var \Ramsey\Collection\Collection<array{string, string, array{body: string}}> $requests */
    $requests = new Collection('array');
    $this->responseFactory = static function (string $method, string $url, array $options) use ($requests): ResponseInterface {
      /** @var array{body: string} $options */
      $requests[] = [$method, $url, $options];

      $response = \Mockery::mock(ResponseInterface::class);
      $response->expects('getContent')
        ->once()
        ->andReturn(\json_encode(['ok' => TRUE, 'ts' => '1234512345.000123', 'channel' => 'CHANNEL']));
      $response->shouldIgnoreMissing();
      $response->expects('getStatusCode')->andReturn(200);

      return $response;
    };

    return $requests;
  }

  private static function notifier(): NotifierInterface {
    return \Drupal::service(NotifierInterface::class);
  }

  private static function sender(): SenderInterface {
    return \Drupal::service(SenderInterface::class);
  }

}
